const questionHelper = require("../Helper/QuestionHelper");
const responseHelper = require("../Helper/ResponseHelper");
const jalaliHelper = require("../Helper/JalaliHelper");

const axios = require("axios")
const dbHelper = require("../Helper/DbHelper");

const PRICE = 10_0000 //Rials

const axiosConfig = {
    headers: {
        'Content-Type': 'application/json',
        'X-API-KEY': process.env.P_SECRET_TOKEN,
    }
}

function createNew(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('INSERT INTO payment (u_id, price) values (?, ?)', [userId, PRICE]))
        .then(dbHelper.getInsertId)
}

async function preparePayment(userId, pId, name, accessToken) {
    let returnCallbackURL = process.env.IS_DEV_MODE.includes("NO_PAYMENT") ? 'http://physiopars.com' : 'https://physiopars.com';
    returnCallbackURL += `/?redirectPage=paymentResult&key=${accessToken}`

    if (process.env.IS_DEV_MODE.includes("NO_PAYMENT")) {
        await createNew(userId)
        return returnCallbackURL
    }

    const orderId = await createNew(userId)
    axios.post('https://api.idpay.ir/v1.1/payment', {
        order_id: orderId,
        amount: PRICE,
        name: name,
        phone: pId,
        mail: '',
        desc: '',
        callback: returnCallbackURL

    }, axiosConfig)
        .then(response => {
            if (process.env.IS_DEV_MODE || true) return returnCallbackURL

            if (response.status !== 201) throw responseHelper.Payment.failToCreate
            else return response.data.link
        })
}

function verifyPayment(userId) {
    let axiosData;
    return new Promise((resolve, reject) =>
        getPaymentInfo(userId)
            .then(paymentInfo => {
                axiosData = {
                    order_id: paymentInfo.orderId,
                    id: paymentInfo.token,
                };
                return axios.post('https://api.idpay.ir/v1.1/payment/inquiry', axiosData, axiosConfig)
            })
            .then(response => {
                if (response.data.status === 10 && false)
                    return axios.post('https://api.idpay.ir/v1.1/payment/verify', axiosData, axiosConfig)
                else {
                    if (process.env.IS_DEV_MODE.includes("NO_PAYMENT")) response.data.status = 200
                    const result = papererResult(response);
                    resolve(result)
                }
                updateMessage(axiosData.order_id, response['payment'])
            })
            .then(response => {
                if (process.env.IS_DEV_MODE.includes("NO_PAYMENT")) response.data.status = 200
                const result = papererResult(response);
                resolve(result)
            })
            .catch(reject));


    function papererResult(response) {
        const validStatuses = [200, 101, 100];
        const responseStatus = response.data.status;

        const isSuccessful = validStatuses.includes(responseStatus) || process.env.IS_DEV_MODE.includes("NO_PAYMENT");

        let paymentInfo = response.data["payment"];
        paymentInfo.date = jalaliHelper.toJalaliDateByTimestamp(paymentInfo["date"])

        const paymentResult = isSuccessful ?
            responseHelper.getPaymentResult(true, paymentInfo["track_id"], paymentInfo["amount"], paymentInfo["date"])
            : responseHelper.getPaymentResult(false, axiosData.order_id, PRICE, Date.now())
        updateStatus(axiosData.order_id, isSuccessful ? 'paid' : 'fail')

        return {isSuccessful, paymentResult}
    }
}

function updateMessage(orderId, message) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('UPDATE payment SET message = ? WHERE id = ? ', [message, orderId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.Payment.notFound))
}

function updateStatus(orderId, newStatus) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('UPDATE payment SET status = ? WHERE id = ? ', [newStatus, orderId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.Payment.notFound))

}

function paymentValidation(userId) {
    if (process.env.IS_DEV_MODE.includes("NO_PAYMENT")) return Promise.resolve()
    return getPaymentByUserId(userId)
        .then(data => {
            const validation = data['validation'] === 'true';
            if (!validation) return Promise.reject(responseHelper.Payment.needToPay)

            return data.id
        })
}

function getPaymentByUserId(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query(`SELECT id, IF(status = 'paid', 'true', 'false') as validation
                            FROM payment
                            WHERE u_id = ?
                            ORDER BY ID DESC
                            LIMIT 1`,
            [userId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.Payment.needToPay))
}

/**
 * @param userId  userID
 * {token,orderId}
 */
function getPaymentInfo(userId) {
    return getPaymentByUserId(userId)
        .then(data => ({token: data.token, orderId: data.id}))
}

async function getPaymentForm(paymentLink) {
    const payment = questionHelper.payment;
    payment.button.url = paymentLink
    payment.price.final = PRICE.toString()
    payment.price.base = PRICE.toString()
    return payment
}


module.exports = {
    preparePayment, verifyPayment, paymentValidation, PRICE, getPaymentForm
};