const responseHelper = require("../Helper/ResponseHelper");
const dbHelper = require("../Helper/DbHelper");

const WAIT_TIME = 72 * 3_600_000 //3 Days

const STATE = {CURED: 'cured', NOT_CURED: 'notCured', IN_PROGRESS: 'inProgress', SUSPEND: 'suspend'}

/*

function isDateBehind(dateString,hours) {
    const hoursAgo = new Date(new Date().getTime() - hours);
    console.log(dateString,hours,hoursAgo,new Date(dateString) < hoursAgo)
    return new Date(dateString) < hoursAgo;
}
*/

function isDateBehind(dateString,hours) {
    console.log("","database:" + new Date(dateString),"current" + new Date(), "72 back" + new Date(dateString) < (new Date() - hours))
    return new Date(dateString) < (new Date() - hours)
}

function create(userId, categoryName) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('INSERT INTO treatment (u_id, t_category) VALUES (?,?)', [userId, categoryName]))
        .then(data => dbHelper.getInsertId(data, responseHelper.Treatment.fail))
}

function updateState(userId, newState, responsible = true) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('UPDATE treatment SET state = ? WHERE u_id = ? AND state = ?', [newState, userId, STATE.IN_PROGRESS]))
        .then(data => dbHelper.mustUpdate(data, responsible ? responseHelper.Treatment.notActive : undefined))
}

function suspendAll(userId) {
    return updateState(userId, STATE.SUSPEND,false)
}

function getState(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('SELECT state, t_category FROM treatment WHERE u_id = ? ORDER BY createTime DESC LIMIT 1', [userId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.Treatment.notFound))
}

function prepareTreatments(userId, categoryName) {
    let questionHelper = require("../Helper/QuestionHelper");
    let gt = questionHelper.GT;
    let children = []
    let categoryFirstPageAttention = questionHelper.Treatment[categoryName].firstPageAttention;
    gt.categoryName = categoryName.startsWith("DT") ? "درمان: تجویز برنامه تمرینات اختصاصی" : "درمان: تجویز برنامه تمرینات عمومی"

    const advanceCategories = ['Advance-Free']
    categoryName.startsWith('DT') && advanceCategories.push('Advance')

    return Promise.all([
        getTreatmentsByCategory(categoryName),
        getTreatmentsByCategory(advanceCategories),
        getTreatmentsForUser(userId)
    ])
        .then(results => {
            children.push({
                id: "VW-1",
                body: categoryFirstPageAttention || "سامانه پس از ارزیابی پاسخ‌های شما تجویز این تمرینات درمانی را به شما پیشنهاد می‌کند.\nخواهشمند است این تمرینات را ۴ تا ۶ بار در روز و هر تمرین را به تعداد ۱۰ تا ۱۵ تکرار انجام دهید.",
                type: "videoWarning",
            })

            let treatment = results[0]
            for (let i = 0; i < treatment.length; i++)
                children.push({
                    id: "GT" + i,
                    url: treatment[i]['resourceURL'],
                    type: "videoBox",
                    header: treatment[i]['header'],
                    subHeader: treatment[i]['subHeader'],
                    thumbnail: '',
                    description: {
                        title: responseHelper.convertToPersianNumber(i + 1) + ') ' + treatment[i].title,
                        body: treatment[i].description
                    }
                })

            children.push({
                id: "VW-2",
                body: "بیماران محترم توصیه اکید ما به حفظ وضعیت صحیح ستون فقرات در شرایط مختلف زندگی و فعالیت‌های روزمره می‌باشد، که در ویدئو‌های بعدی به نمایش گذاشته شده است.",
                type: "videoWarning",
            })

            let advances = results[1]
            for (let i = 0; i < advances.length; i++)
                children.push({
                    id: "ET" + i,
                    url: advances[i]['resourceURL'],
                    type: "videoBox",
                    header: advances[i]['header'],
                    subHeader: advances[i]['subHeader'],
                    thumbnail: '',
                    description: {
                        title: responseHelper.convertToPersianNumber(i + 1) + ') ' + advances[i].title,
                        body: advances[i].description
                    }
                })

            children.push({
                id: "VW-3",
                body: "خواهشمندیم بعد از اجرای تمرینات و رعایت اصول ارگونومیک نتیجه درمان (کاهش/افزایش درد) را ۷۲ ساعت بعد از اجرا، در سامانه ثبت فرمایید تا مرحله بعدی درمان شما مشخص شود.",
                type: "videoWarning",
            })

            gt.children = children

            let localTreatment = results[2]
            gt.enabledCuredBox = process.env.IS_DEV_MODE.includes("NO_72_WAIT") || isDateBehind(localTreatment.createTime, WAIT_TIME)
            return gt
        })
}

function getTreatmentsByCategory(category) {
    let query = [], parameters = []

    if (Array.isArray(category)) {
        for (let c = 0; c < category.length; c++) query.push('category = ? ')
        query = query.join('Or ')
        parameters.push(...category)
    } else {
        query = 'category = ?'
        parameters.push(category)
    }

    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query(
            `SELECT *
             FROM treatment_const
             WHERE ${query}
             ORDER BY treatment_const.order`,
            parameters))
        .then(dbHelper.getArray)
}

function getTreatmentsForUser(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('SELECT * FROM treatment  WHERE u_id = ? AND state = ? LIMIT 1', [userId, STATE.IN_PROGRESS]))
        .then(data => dbHelper.getSingleton(data, responseHelper.Treatment.notFound))
        .then(data => dbHelper.getSingleton(data, responseHelper.Treatment.notFound))
}

function createTreatment(userId, userStep) {
    return suspendAll(userId)
        .then(() => create(userId, userStep))
        .then(() => prepareTreatments(userId,userStep))
        .then(gt => {
            gt.enabledCuredBox = false; // when treatment created by assessment, user can't access this
            return gt
        })
}

module.exports = {
    getState, updateState, STATE, prepareTreatments, createTreatment
};