const responseHelper = require("../Helper/ResponseHelper");
const dbHelper = require("../Helper/DbHelper")

function isInLess2Min(startDate) {
    startDate = new Date(startDate);
    const diffInMilliseconds = new Date().getTime() - startDate.getTime();
    return Math.round(diffInMilliseconds / 60_000) < 2; // 1 minute = 60,000 milliseconds
}

function createSimpleOTP() {
    const crypto = require('crypto');

// Define an array of number patterns that are easy to remember
    const numberPatterns = [
        [0, 1, 2, 3, 4, 5],
        [1, 2, 3, 4, 5, 6],
        [2, 3, 4, 5, 6, 7],
        [3, 4, 5, 6, 7, 8],
        [4, 5, 6, 7, 8, 9],
        [5, 6, 7, 8, 9, 0],
        [6, 7, 8, 9, 0, 1],
        [7, 8, 9, 0, 1, 2],
        [8, 9, 0, 1, 2, 3],
        [9, 0, 1, 2, 3, 4]
    ];

// Generate a random index into the number pattern array
    const patternIndex = crypto.randomInt(0, numberPatterns.length);

// Generate a random starting number within the pattern
    let startingNumber = crypto.randomInt(0, 9);

// Create the password by iterating over the pattern and applying it to the starting number
    let password = '';
    const pattern = numberPatterns[patternIndex];
    for (let i = 0; i < 6; i++) {
        password += startingNumber;
        startingNumber = pattern[startingNumber > 5 ? startingNumber - 6 : startingNumber];
    }
    return password;

}

function generateNewOTP(id) {
    return dbHelper.dbConnection
        .then(dbConnection => dbConnection.query('SELECT otpTime FROM users WHERE id = ?', [id]))
        .then(data => dbHelper.getSingleton(data, responseHelper.User.notFound))
        .then(data => {
            const otpTime = data['otpTime'];
            if (otpTime && isInLess2Min(otpTime)) return Promise.reject(responseHelper.OTP.tooEarly)

            const otp = createSimpleOTP();
            updateOTP(id, otp)
            return otp
        })
}

function updateStep(id, newStep) {
    return isUserVerified(id)
        .then(() => dbHelper.dbConnection)
        .then(dbConnection => dbConnection.query('UPDATE users SET step = ? WHERE id = ?', [newStep, id]))
        .then(data => dbHelper.mustUpdate(data, responseHelper.User.notFound))
        .then(() => newStep)
}

function updateOTP(id, otp) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('UPDATE users SET otp = ?, otpTime = CURRENT_TIMESTAMP() WHERE id = ?', [otp, id]))
}

function verifyOTP(userId, otp) {
    if (process.env.IS_DEV_MODE.includes("NO_OTP"))
        return updateOTP(userId, undefined)
    else
        return dbHelper.dbConnection
            .then(dbConnection =>
                dbConnection.query(`SELECT IF(otp = ?, 'true', 'false') as OTPVerified
                                    FROM users
                                    WHERE id = ?`, [otp, userId]))
            .then(data => dbHelper.getSingleton(data, responseHelper.User.notFound))
            .then(data => {
                if (data['OTPVerified'] !== 'true') return Promise.reject(responseHelper.OTP.fail)
                updateOTP(userId, undefined)
            })
}

//PROTOCOL_ENQUEUE_AFTER_FATAL_ERROR

function verifyPId(pId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('SELECT id FROM users WHERE pId = ?', [pId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.User.notFound))
        .then(data => data.id)
}

function creatUser(pId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('INSERT INTO users (pId) VALUES (?)', [pId]))
        .then(data => dbHelper.getInsertId(data, responseHelper.User.noValidPhone))
        .then(userId => {
            let autHelper = require("../Helper/AuthHelper");
            let refToken = autHelper.getRefreshToken(userId);
            updateRefreshToken(userId, refToken)
            return userId
        })

}

function updateRefreshToken(id, refToken) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('UPDATE users SET refToken = ? WHERE id = ?', [refToken, id]))
}

function isUserVerified(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query(`SELECT 'true' as OTPverfiy
                            FROM users
                            WHERE id = ?
                              AND otp IS NULL`, [userId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.User.notVerified))
}

function getUserStep(userId) {
    return isUserVerified(userId)
        .then(() => dbHelper.dbConnection)
        .then(dbConnection => dbConnection.query('SELECT step FROM users WHERE id = ?', [userId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.User.notFound))
        .then(data => data.step)
}

function getUserPId(userId) {
    return dbHelper.dbConnection.then(dbConnection =>
        dbConnection.query('SELECT pId FROM users WHERE id = ?', [userId]))
        .then(data => dbHelper.getSingleton(data, responseHelper.User.notFound))
        .then(data => data.pId)
}

module.exports = {
    updateStep,generateNewOTP,
    verifyOTP, verifyPId,
    creatUser, getUserStep, getUserPId
};