const responseHelper = require("./ResponseHelper");

/**
 * replace empty array with default values
 * @param inputs the given input
 * @returns {*} the given input also with default values
 */
function verifyEmptyArray(inputs) {
    const validate = [
        {input: 'p5m9', default: "p5m9o0"},
        {input: 'p5m10', default: "p5m10o0"},
        {input: 'p5m11', default: "p5m11o0"},
        {input: 'p5m12', default: "p5m12o0"}
    ]
    for (const validateCondition of validate) {
        if (inputs[validateCondition.input]) {
            if (inputs[validateCondition.input].length) continue
            inputs[validateCondition.input] = [validateCondition.default]
        }
    }
    return inputs;
}

function validateInputsValues(inputs) {
    let validate;

    if (inputs['p1m1']) {
        validate = validateTextarea(inputs['p1m1']);
        if (validate) return validate
    }

    if (inputs['p1m2']) {
        validate = validateTextarea(inputs['p1m2']);
        if (validate) return validate
    }

    if (inputs['p1m8']) {
        validate = validateEmail(inputs['p1m8']);
        if (validate) return validate
    }

    if (inputs['p1m7']) if (!isBetween(inputs['p1m7'], 0, 10)) return responseHelper.InvalidNumber + " The input value is: " + inputs['p1m7']

    if (inputs['p1m5']) if (!isBetween(inputs['p1m5'], 10, 100)) return responseHelper.InvalidNumber + " The input value is: " + inputs['p1m5']

    const skipElements = ["p1m1", "p1m2", "p1m8", "p1m7", "p1m5", 'pwm1']

    const check = [];
    for (const inputsKey in inputs) {
        if (!skipElements.includes(inputsKey))
            check.push({input: inputs[inputsKey], condition: connector[inputsKey]});
    }


    try {
        validate = checkSingleItem(check)
        if (validate) return validate
    } catch (e) {
        return 'Invalid inputs on: ' + JSON.stringify(inputs)
    }

    return undefined
}

function checkSingleItem(arr) {
    for (const arrElement of arr) {
        if (arrElement.input) {
            if (Array.isArray(arrElement.input))
                for (const inputElement of arrElement.input) {
                    if (!arrElement.condition.includes(inputElement))
                        return responseHelper.InvalidInput + " The input option is: " + arrElement.input
                }
            else if (!arrElement.condition.includes(arrElement.input))
                return responseHelper.InvalidInput + " The input option is: " + arrElement.input
        }
    }
}

function validateTextarea(str) {
    if (hasInvalidChars(str)) return responseHelper.InvalidCharacters + " The input value is: " + str
    else if (str.length > 100) return responseHelper.InvalidLength + " The input value is: " + str
    else return undefined;
}

function hasInvalidChars(str) {
    const invalidChars = /[()\[\]{}.a-zA-Z]/g;
    return invalidChars.test(str);
}

function validateEmail(email) {
    const regex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
    if (regex.test(email)) {
        return false
    } else return responseHelper.InvalidEmail + " The input value is: " + email
}

function isBetween(num, min, max) {
    return num >= min && num <= max;
}

function validatePhoneNumber(phone) {
    const re = /^\(?(\d{3})\)?[- ]?(\d{3})[- ]?(\d{4})$/;
    return re.test(phone);
}
function validateGeneralAssessment(assessment) {
        return new Promise((resolve, reject) => {
            if (assessment['p2m1'] && !assessment['p2m1'].length) return reject(responseHelper.Assessment.p2Empty)
            resolve()
        })
}


const connector = {
    p1m3: ["p1m3o1", "p1m3o2", "p1m3o3", "p1m3o4", "p1m3o5"],
    p1m4: ["p1m4o1", "p1m4o2", "p1m4o3", "p1m4o4"],
    p1m6: ["p1m6o1", "p1m6o2"],

    p2m1: ['o18', 'o15', 'o14', 'o17', 'o16', 'o13', 'o12', 'o7', 'o2', 'o1', 'o3', 'o8', 'o9', 'o4', 'o5', 'o10', 'o11', 'o6'],
    p3m1: ["p3m1o1", "p3m1o2", "p3m1o3", "p3m1o4", "p3m1o5", "p3m1o6"],
    p3m2: ["p3m2o1", "p3m2o2", "p3m2o3", "p3m2o4", "p3m2o5"],

    p4m1: [1, 0], p4m2: [1, 0], p4m3: [1, 0],
    p4m4: [1, 0], p4m5: [1, 0], p4m6: [1, 0],
    p4m7: [1, 0], p4m8: [1, 0],
    p4m9: ["p4m9o1", "p4m9o2", "p4m9o3", "p4m9o4", "p4m9o5", "p4m9o6"],
    p4m10: [1, 0],

    p5m1: [1, 0], p5m2: [1, 0], p5m3: [1, 0],
    p5m4: [1, 0], p5m5: [1, 0], p5m6: [1, 0],
    p5m7: [1, 0], p5m8: [1, 0], p5m13: [1, 0],
    p5m9: ["p5m9o0", "p5m9o1", "p5m9o2", "p5m9o3", "p5m9o4", "p5m9o5", "p5m9o6", "p5m9o7", "p5m9o8"],
    p5m10: ["p5m10o0", "p5m10o1", "p5m10o2", "p5m10o3", "p5m10o4", "p5m10o5"],
    p5m11: ["p5m11o0", "p5m11o1", "p5m11o2", "p5m11o3", "p5m11o4", "p5m11o5"],
    p5m12: ["p5m12o0", "p5m12o1", "p5m12o2", "p5m12o3", "p5m12o4", "p5m12o5"],

    d1: ["d1o1", "d1o2"],
    d2: ["d2o1", "d2o2", "d2o3"],
    d3: ["d3o1", "d3o2", "d3o3"],
    d4: ["d4o1", "d4o2", "d4o3"],
    d5: ["d5o1", "d5o2", "d5o3"],
    d6: ["d6o1", "d6o2", "d6o3", "d6o4"],
    d7: ["d7o1", "d7o2", "d7o3"],
    d8: ["d8o1", "d8o2", "d8o3"],
    d9: ["d9o1", "d9o2", "d9o3"],
    d10: ["d10o1", "d10o2", "d10o3"],
    d11: ["d11o1", "d11o2", "d11o3"],
    d12: ["d12o1", "d12o2", "d12o3"],
    d13: ["d13o1", "d13o2", "d13o3"],
    d14: ["d14o1", "d14o2", "d14o3"],
    d15: ["d15o1", "d15o2", "d15o3"],
    d16: ["d16o1", "d16o2", "d16o3"],
    d17: ["d17o1", "d17o2", "d17o3"],
    d18: ["d18o1", "d18o2", "d18o3"],
    d19: ["d19o1", "d19o2", "d19o3"],

}

module.exports = {
    validateInputsValues, validatePhoneNumber, verifyEmptyArray,validateGeneralAssessment
}