const express = require('express');
const path = require('path');
const http = require("http");
const cors = require("cors");
const fs = require("fs");

require('dotenv').config()

const app = express();

app.use(express.json());
app.use(express.urlencoded({extended: false}));
app.use(express.static(path.join(__dirname, 'public')));
app.use(cors())

if (!process.env.IS_DEV_MODE.includes("NO_LOG")) {
    process.on('uncaughtException', err => require('fs').appendFileSync('./debug.log', 'caughtException: ' + err.stack + '\r\n'))
    /*console.log = (...arg)=>fs.appendFileSync('./debug.log', 'path: ' + arg.join( '\r\n'))

    app.use((req, res, next) => {
        let fs = require('fs');
        fs.appendFileSync('./debug.log', 'path: ' + req.path + '\r\n')
        fs.appendFileSync('./debug.log', 'body: ' + JSON.stringify(req.body) + '\n----------\n')

        console.log(req.path);
        console.log(req.body)
        next();
    })*/
}

app.use(replaceInsecureCharacters)

const baseURL = process.env.BASE_URL;
app.use(baseURL + '/', require('./routes/index'));
app.use(baseURL + '/login', require('./routes/login'));
app.use(baseURL + '/assessments', require('./routes/assessments'));
app.use(baseURL + '/treatment', require('./routes/traetment'));
app.use(baseURL + '/payment', require('./routes/payment'));

function replaceInsecureCharacters(req, res, next) {
    const insecureCharsRegex = /[<>&'"\/\\]/g; // Regular expression to match insecure characters

    // Iterate over each property in the request body
    // Remove insecure characters from the string
    if (req.body) for (const key in req.body)
        if (typeof req.body[key] === 'string') req.body[key] = req.body[key].replace(insecureCharsRegex, '');

    // Iterate over each query parameter
    // Remove insecure characters from the string
    if (req.query) for (const key in req.query)
        if (typeof req.query[key] === 'string') req.query[key] = req.query[key].replace(insecureCharsRegex, '');

    next();
}


http.createServer(app).listen(3000);