const express = require('express');
const router = express.Router();

const authHelper = require("../Helper/AuthHelper");
const questionHelper = require('../Helper/QuestionHelper')
const validatorHelper = require('../Helper/ValidatorHelper')
const docHelper = require('../Helper/DocHelper')
const responseHelper = require("../Helper/ResponseHelper");
const nextFormHelper = require("../Helper/NextFormHelper");

const userController = require("../Controller/UserController");
const assessmentsController = require("../Controller/AssessmentsController");
const treatmentController = require("../Controller/TreatmentController");
const relationController = require("../Controller/RelationController");
const paymentController = require("../Controller/PaymentController");
const sMSController = require("../Controller/SMSController");

router.post("/requestNext", authHelper.auth, (req, res) => {
    const body = req.body.inputs;
    const userId = req.userId;

    const validateMessage = validatorHelper.validateInputsValues(body);
    if (validateMessage) return res.json({success: false, message: validateMessage, data: {}})
    const inputs = validatorHelper.verifyEmptyArray(body)

    userController.getUserStep(userId)
        .then(userStep => {

            if (userStep.startsWith('p'))
                return validatorHelper.validateGeneralAssessment(body)
                    .then(() => assessmentsController.insertOrUpdate(userId, userStep, inputs))
                    .then(assesId => assessmentsController.getAssessmentsById(assesId))
                    .then(assessment => {
                        userStep = relationController.findNextGeneralStep(assessment, userStep);
                        userController.updateStep(userId, userStep)
                        prepareUserStepResponse(assessment)
                    })
            else if (userStep.startsWith('PW')) {
                if (body['pwm1'] && body['pwm1'] == '1')
                    return assessmentsController.getAssessment(userId).then(assessment => {
                        userStep = relationController.findNextGeneralStep(assessment, userStep);
                        userController.updateStep(userId, userStep)
                        prepareUserStepResponse(assessment);
                    })
                else onRejected(responseHelper.Assessment.pregnancy)
            } else if (userStep.startsWith('d'))
                return assessmentsController.getAssessment(userId)
                    .then(assessment => {
                        assessment = Object.assign(assessment, body);
                        body.lastUpdateUserStep = userStep;
                        userStep = relationController.findNextDedicateStep(assessment, userStep);
                        assessmentsController.update(assessment.id, body)
                        userController.updateStep(userId, userStep)
                        prepareUserStepResponse(assessment)
                    })
            else if (userStep.startsWith('P'))
                return paymentController.paymentValidation(userId)
                    .then(() => nextFormHelper.nextOfTreatment(userId))
                    .then(onResponse)


            /**
             * by userStep, manage Treatment Or RedFlag Or Assessment
             * @param assessment the Assessment
             */
            function prepareUserStepResponse(assessment) {
                if (userStep.startsWith("RF"))
                    userController.getUserPId(userId)
                        .then(pId => docHelper.create(pId, assessment))
                        .then(pdfUrl => {
                            questionHelper.RF.buttons[0].url = pdfUrl;
                            onResponse(questionHelper.RF)
                        }).catch(onRejected)
                else if (userStep.startsWith("GT"))
                    userController.getUserPId(userId)
                        .then(pId => sMSController.GT(pId, relationController.getName(assessment)))
                        .then(() => treatmentController.createTreatment(userId, userStep))
                        .then(onResponse).catch(onRejected)
                else if (userStep.startsWith("NTV"))
                    onResponse(questionHelper.NTV)
                else if (userStep.startsWith("p") || userStep.startsWith('PW'))
                    assessmentsController.fillGeneralAnswers(userStep, userId)
                        .then(ga => {
                            ga.registeredLabel = relationController.getName(assessment);
                            onResponse(ga)
                        })
                        .catch(onRejected)
                else if (userStep.startsWith("DT"))
                    treatmentController.createTreatment(userId, userStep).then(onResponse).catch(onRejected)
                else if (userStep.startsWith("d"))
                    assessmentsController.fillDedicatedAnswer(userStep, userId).then(onResponse).catch(onRejected)
            }
        })
        .catch(onRejected);

    function onResponse(data) {
        data.registeredLabel ??= "کاربر جدید"
        res.json({success: true, message: "", data})
    }

    function onRejected(error) {
        res.json({success: false, message: error, data: {}})
    }

})


router.post("/requestBack", authHelper.auth, (req, res) => {
    const userId = req.userId;
    const minStepResponse = {success: false, message: responseHelper.MinStep, data: []};
    let assessment;
    assessmentsController.getAssessment(userId)
        .then(_assessment => {
            assessment = _assessment
            return userController.getUserStep(userId)
        })
        .then(userStep => {
            if (userStep.startsWith('p')) {
                userStep = relationController.findBackGeneralStep(userStep)
                if (userStep)
                    return userController.updateStep(userId, userStep)
                        .then(() => assessmentsController.fillGeneralAnswers(userStep, userId))
                        .then(element => res.json({success: true, message: "", data: element}))
                else
                    return res.json(minStepResponse)
            } else if (userStep.startsWith('GT'))
                return res.json(minStepResponse)
            else if (userStep.startsWith('d')) {
                userStep = relationController.findBackDedicateStep(assessment, userStep)
                if (userStep)
                    return userController.updateStep(userId, userStep)
                        .then(() => assessmentsController.fillDedicatedAnswer(userStep, userId))
                        .then(element => res.json({success: true, message: "", data: element}))
                else res.json(minStepResponse)
            } else res.json(minStepResponse)
        })
        .catch(onRejected)

    function onRejected(error) {
        res.json({success: false, message: error, data: {}})
    }

});


module.exports = router;